from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.transition_event import TransitionEvent
from ...types import UNSET, Response


def _get_kwargs(
    id: str,
    *,
    next_token: Union[None, str],
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    json_next_token: Union[None, str]
    json_next_token = next_token
    params["next_token"] = json_next_token

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/executions/{id}/transitions.stream".format(
            id=id,
        ),
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[TransitionEvent]:
    if response.status_code == HTTPStatus.OK:
        response_200 = TransitionEvent.from_dict(response.text)

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[TransitionEvent]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    next_token: Union[None, str],
) -> Response[TransitionEvent]:
    """Stream events emitted by the given execution

    Args:
        id (str):
        next_token (Union[None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[TransitionEvent]
    """

    kwargs = _get_kwargs(
        id=id,
        next_token=next_token,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    next_token: Union[None, str],
) -> Optional[TransitionEvent]:
    """Stream events emitted by the given execution

    Args:
        id (str):
        next_token (Union[None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        TransitionEvent
    """

    return sync_detailed(
        id=id,
        client=client,
        next_token=next_token,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    next_token: Union[None, str],
) -> Response[TransitionEvent]:
    """Stream events emitted by the given execution

    Args:
        id (str):
        next_token (Union[None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[TransitionEvent]
    """

    kwargs = _get_kwargs(
        id=id,
        next_token=next_token,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    id: str,
    *,
    client: Union[AuthenticatedClient, Client],
    next_token: Union[None, str],
) -> Optional[TransitionEvent]:
    """Stream events emitted by the given execution

    Args:
        id (str):
        next_token (Union[None, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        TransitionEvent
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            next_token=next_token,
        )
    ).parsed



import datetime
from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="ResourceDeletedResponse")


@_attrs_define
class ResourceDeletedResponse:
    """
    Attributes:
        id (str):
        deleted_at (datetime.datetime): When this resource was deleted as UTC date-time
        jobs (List[str]): IDs (if any) of jobs created as part of this request
    """

    id: str
    deleted_at: datetime.datetime
    jobs: List[str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        deleted_at = self.deleted_at.isoformat()

        jobs = self.jobs

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "deleted_at": deleted_at,
                "jobs": jobs,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        deleted_at = isoparse(d.pop("deleted_at"))

        jobs = cast(List[str], d.pop("jobs"))

        resource_deleted_response = cls(
            id=id,
            deleted_at=deleted_at,
            jobs=jobs,
        )

        resource_deleted_response.additional_properties = d
        return resource_deleted_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="BaseTokenLogProb")


@_attrs_define
class BaseTokenLogProb:
    """
    Attributes:
        token (str):
        logprob (float): The log probability of the token
        bytes_ (Union[Unset, List[int]]):
    """

    token: str
    logprob: float
    bytes_: Union[Unset, List[int]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        token = self.token

        logprob = self.logprob

        bytes_: Union[Unset, List[int]] = UNSET
        if not isinstance(self.bytes_, Unset):
            bytes_ = self.bytes_

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "token": token,
                "logprob": logprob,
            }
        )
        if bytes_ is not UNSET:
            field_dict["bytes"] = bytes_

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        token = d.pop("token")

        logprob = d.pop("logprob")

        bytes_ = cast(List[int], d.pop("bytes", UNSET))

        base_token_log_prob = cls(
            token=token,
            logprob=logprob,
            bytes_=bytes_,
        )

        base_token_log_prob.additional_properties = d
        return base_token_log_prob

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.task_token_resume_execution_request_status import (
    TaskTokenResumeExecutionRequestStatus,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.task_token_resume_execution_request_input import (
        TaskTokenResumeExecutionRequestInput,
    )


T = TypeVar("T", bound="TaskTokenResumeExecutionRequest")


@_attrs_define
class TaskTokenResumeExecutionRequest:
    """
    Attributes:
        status (TaskTokenResumeExecutionRequestStatus):  Default: TaskTokenResumeExecutionRequestStatus.RUNNING.
        input_ (Union[Unset, TaskTokenResumeExecutionRequestInput]): The input to resume the execution with
    """

    status: TaskTokenResumeExecutionRequestStatus = (
        TaskTokenResumeExecutionRequestStatus.RUNNING
    )
    input_: Union[Unset, "TaskTokenResumeExecutionRequestInput"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.task_token_resume_execution_request_input import (
            TaskTokenResumeExecutionRequestInput,
        )

        status = self.status.value

        input_: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.input_, Unset):
            input_ = self.input_.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
            }
        )
        if input_ is not UNSET:
            field_dict["input"] = input_

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.task_token_resume_execution_request_input import (
            TaskTokenResumeExecutionRequestInput,
        )

        d = src_dict.copy()
        status = TaskTokenResumeExecutionRequestStatus(d.pop("status"))

        _input_ = d.pop("input", UNSET)
        input_: Union[Unset, TaskTokenResumeExecutionRequestInput]
        if isinstance(_input_, Unset):
            input_ = UNSET
        else:
            input_ = TaskTokenResumeExecutionRequestInput.from_dict(_input_)

        task_token_resume_execution_request = cls(
            status=status,
            input_=input_,
        )

        task_token_resume_execution_request.additional_properties = d
        return task_token_resume_execution_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.function_def_parameters import FunctionDefParameters


T = TypeVar("T", bound="FunctionDef")


@_attrs_define
class FunctionDef:
    """Function definition

    Attributes:
        name (Union[Unset, Any]): DO NOT USE: This will be overriden by the tool name. Here only for compatibility
            reasons.
        description (Union[Unset, str]): For Unicode character safety
            See: https://unicode.org/reports/tr31/
            See: https://www.unicode.org/reports/tr39/#Identifier_Characters
        parameters (Union[Unset, FunctionDefParameters]): The parameters the function accepts
    """

    name: Union[Unset, Any] = UNSET
    description: Union[Unset, str] = UNSET
    parameters: Union[Unset, "FunctionDefParameters"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.function_def_parameters import FunctionDefParameters

        name = self.name

        description = self.description

        parameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.parameters, Unset):
            parameters = self.parameters.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if parameters is not UNSET:
            field_dict["parameters"] = parameters

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.function_def_parameters import FunctionDefParameters

        d = src_dict.copy()
        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        _parameters = d.pop("parameters", UNSET)
        parameters: Union[Unset, FunctionDefParameters]
        if isinstance(_parameters, Unset):
            parameters = UNSET
        else:
            parameters = FunctionDefParameters.from_dict(_parameters)

        function_def = cls(
            name=name,
            description=description,
            parameters=parameters,
        )

        function_def.additional_properties = d
        return function_def

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.finish_reason import FinishReason
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.chat_output_chunk_delta import ChatOutputChunkDelta
    from ..models.log_prob_response import LogProbResponse


T = TypeVar("T", bound="ChatOutputChunk")


@_attrs_define
class ChatOutputChunk:
    """Streaming chat completion output

    Attributes:
        index (int):
        finish_reason (FinishReason): The reason the model stopped generating tokens. This will be `stop`
            if the model hit a natural stop point or a provided stop sequence,
            `length` if the maximum number of tokens specified in the request
            was reached, `content_filter` if content was omitted due to a flag
            from our content filters, `tool_calls` if the model called a tool. Default: FinishReason.STOP.
        delta (ChatOutputChunkDelta): The message generated by the model
        logprobs (Union[Unset, LogProbResponse]):
    """

    index: int
    delta: "ChatOutputChunkDelta"
    finish_reason: FinishReason = FinishReason.STOP
    logprobs: Union[Unset, "LogProbResponse"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.chat_output_chunk_delta import ChatOutputChunkDelta
        from ..models.log_prob_response import LogProbResponse

        index = self.index

        finish_reason = self.finish_reason.value

        delta = self.delta.to_dict()

        logprobs: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.logprobs, Unset):
            logprobs = self.logprobs.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "index": index,
                "finish_reason": finish_reason,
                "delta": delta,
            }
        )
        if logprobs is not UNSET:
            field_dict["logprobs"] = logprobs

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.chat_output_chunk_delta import ChatOutputChunkDelta
        from ..models.log_prob_response import LogProbResponse

        d = src_dict.copy()
        index = d.pop("index")

        finish_reason = FinishReason(d.pop("finish_reason"))

        delta = ChatOutputChunkDelta.from_dict(d.pop("delta"))

        _logprobs = d.pop("logprobs", UNSET)
        logprobs: Union[Unset, LogProbResponse]
        if isinstance(_logprobs, Unset):
            logprobs = UNSET
        else:
            logprobs = LogProbResponse.from_dict(_logprobs)

        chat_output_chunk = cls(
            index=index,
            finish_reason=finish_reason,
            delta=delta,
            logprobs=logprobs,
        )

        chat_output_chunk.additional_properties = d
        return chat_output_chunk

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.doc_reference import DocReference


T = TypeVar("T", bound="DocSearchResponse")


@_attrs_define
class DocSearchResponse:
    """
    Attributes:
        docs (List['DocReference']): The documents that were found
        time (float): The time taken to search in seconds
    """

    docs: List["DocReference"]
    time: float
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.doc_reference import DocReference

        docs = []
        for docs_item_data in self.docs:
            docs_item = docs_item_data.to_dict()
            docs.append(docs_item)

        time = self.time

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "docs": docs,
                "time": time,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.doc_reference import DocReference

        d = src_dict.copy()
        docs = []
        _docs = d.pop("docs")
        for docs_item_data in _docs:
            docs_item = DocReference.from_dict(docs_item_data)

            docs.append(docs_item)

        time = d.pop("time")

        doc_search_response = cls(
            docs=docs,
            time=time,
        )

        doc_search_response.additional_properties = d
        return doc_search_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class BaseEntrySource(str, Enum):
    API_REQUEST = "api_request"
    API_RESPONSE = "api_response"
    INTERNAL = "internal"
    META = "meta"
    SUMMARIZER = "summarizer"
    TOOL_RESPONSE = "tool_response"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ToolType(str, Enum):
    API_CALL = "api_call"
    FUNCTION = "function"
    INTEGRATION = "integration"
    SYSTEM = "system"

    def __str__(self) -> str:
        return str(self.value)



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.base_entry_content_type_0_item_type_1_type import (
    BaseEntryContentType0ItemType1Type,
)
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.base_entry_content_type_0_item_type_1_image_url import (
        BaseEntryContentType0ItemType1ImageUrl,
    )


T = TypeVar("T", bound="BaseEntryContentType0ItemType1")


@_attrs_define
class BaseEntryContentType0ItemType1:
    """
    Attributes:
        image_url (BaseEntryContentType0ItemType1ImageUrl): The image URL
        type (BaseEntryContentType0ItemType1Type): The type (fixed to 'image_url') Default:
            BaseEntryContentType0ItemType1Type.IMAGE_URL.
    """

    image_url: "BaseEntryContentType0ItemType1ImageUrl"
    type: BaseEntryContentType0ItemType1Type = (
        BaseEntryContentType0ItemType1Type.IMAGE_URL
    )
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.base_entry_content_type_0_item_type_1_image_url import (
            BaseEntryContentType0ItemType1ImageUrl,
        )

        image_url = self.image_url.to_dict()

        type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "image_url": image_url,
                "type": type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.base_entry_content_type_0_item_type_1_image_url import (
            BaseEntryContentType0ItemType1ImageUrl,
        )

        d = src_dict.copy()
        image_url = BaseEntryContentType0ItemType1ImageUrl.from_dict(d.pop("image_url"))

        type = BaseEntryContentType0ItemType1Type(d.pop("type"))

        base_entry_content_type_0_item_type_1 = cls(
            image_url=image_url,
            type=type,
        )

        base_entry_content_type_0_item_type_1.additional_properties = d
        return base_entry_content_type_0_item_type_1

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.vector_doc_search_request_lang import VectorDocSearchRequestLang
from ..types import UNSET, Unset

T = TypeVar("T", bound="VectorDocSearchRequest")


@_attrs_define
class VectorDocSearchRequest:
    """
    Attributes:
        limit (int):  Default: 10.
        lang (VectorDocSearchRequestLang): The language to be used for text-only search. Support for other languages
            coming soon. Default: VectorDocSearchRequestLang.EN_US.
        confidence (float): The confidence cutoff level Default: 0.5.
        vector (List[float]): Vector to use in the search. Must be the same dimensions as the embedding model or else an
            error will be thrown.
    """

    vector: List[float]
    limit: int = 10
    lang: VectorDocSearchRequestLang = VectorDocSearchRequestLang.EN_US
    confidence: float = 0.5
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        limit = self.limit

        lang = self.lang.value

        confidence = self.confidence

        vector = self.vector

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "limit": limit,
                "lang": lang,
                "confidence": confidence,
                "vector": vector,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        limit = d.pop("limit")

        lang = VectorDocSearchRequestLang(d.pop("lang"))

        confidence = d.pop("confidence")

        vector = cast(List[float], d.pop("vector"))

        vector_doc_search_request = cls(
            limit=limit,
            lang=lang,
            confidence=confidence,
            vector=vector,
        )

        vector_doc_search_request.additional_properties = d
        return vector_doc_search_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="DefaultChatSettings")


@_attrs_define
class DefaultChatSettings:
    """Default settings for the chat session (also used by the agent)

    Attributes:
        frequency_penalty (Union[Unset, float]): Number between -2.0 and 2.0. Positive values penalize new tokens based
            on their existing frequency in the text so far, decreasing the model's likelihood to repeat the same line
            verbatim.
        presence_penalty (Union[Unset, float]): Number between -2.0 and 2.0. Positive values penalize new tokens based
            on their existing frequency in the text so far, decreasing the model's likelihood to repeat the same line
            verbatim.
        temperature (Union[Unset, float]): What sampling temperature to use, between 0 and 2. Higher values like 0.8
            will make the output more random, while lower values like 0.2 will make it more focused and deterministic.
        top_p (Union[Unset, float]): Defaults to 1 An alternative to sampling with temperature, called nucleus sampling,
            where the model considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens
            comprising the top 10% probability mass are considered.  We generally recommend altering this or temperature but
            not both.
        repetition_penalty (Union[Unset, float]): Number between 0 and 2.0. 1.0 is neutral and values larger than that
            penalize new tokens based on their existing frequency in the text so far, decreasing the model's likelihood to
            repeat the same line verbatim.
        length_penalty (Union[Unset, float]): Number between 0 and 2.0. 1.0 is neutral and values larger than that
            penalize number of tokens generated.
        min_p (Union[Unset, float]): Minimum probability compared to leading token to be considered
    """

    frequency_penalty: Union[Unset, float] = UNSET
    presence_penalty: Union[Unset, float] = UNSET
    temperature: Union[Unset, float] = UNSET
    top_p: Union[Unset, float] = UNSET
    repetition_penalty: Union[Unset, float] = UNSET
    length_penalty: Union[Unset, float] = UNSET
    min_p: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        frequency_penalty = self.frequency_penalty

        presence_penalty = self.presence_penalty

        temperature = self.temperature

        top_p = self.top_p

        repetition_penalty = self.repetition_penalty

        length_penalty = self.length_penalty

        min_p = self.min_p

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if frequency_penalty is not UNSET:
            field_dict["frequency_penalty"] = frequency_penalty
        if presence_penalty is not UNSET:
            field_dict["presence_penalty"] = presence_penalty
        if temperature is not UNSET:
            field_dict["temperature"] = temperature
        if top_p is not UNSET:
            field_dict["top_p"] = top_p
        if repetition_penalty is not UNSET:
            field_dict["repetition_penalty"] = repetition_penalty
        if length_penalty is not UNSET:
            field_dict["length_penalty"] = length_penalty
        if min_p is not UNSET:
            field_dict["min_p"] = min_p

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        frequency_penalty = d.pop("frequency_penalty", UNSET)

        presence_penalty = d.pop("presence_penalty", UNSET)

        temperature = d.pop("temperature", UNSET)

        top_p = d.pop("top_p", UNSET)

        repetition_penalty = d.pop("repetition_penalty", UNSET)

        length_penalty = d.pop("length_penalty", UNSET)

        min_p = d.pop("min_p", UNSET)

        default_chat_settings = cls(
            frequency_penalty=frequency_penalty,
            presence_penalty=presence_penalty,
            temperature=temperature,
            top_p=top_p,
            repetition_penalty=repetition_penalty,
            length_penalty=length_penalty,
            min_p=min_p,
        )

        default_chat_settings.additional_properties = d
        return default_chat_settings

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.base_chat_output import BaseChatOutput
    from ..models.competion_usage import CompetionUsage
    from ..models.doc_reference import DocReference


T = TypeVar("T", bound="MessageChatResponse")


@_attrs_define
class MessageChatResponse:
    """
    Attributes:
        choices (List['BaseChatOutput']): The deltas generated by the model
        jobs (List[str]): Background job IDs that may have been spawned from this interaction.
        docs (List['DocReference']): Documents referenced for this request (for citation purposes).
        created_at (datetime.datetime): When this resource was created as UTC date-time
        id (str):
        usage (Union[Unset, CompetionUsage]): Usage statistics for the completion request
    """

    choices: List["BaseChatOutput"]
    jobs: List[str]
    docs: List["DocReference"]
    created_at: datetime.datetime
    id: str
    usage: Union[Unset, "CompetionUsage"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.base_chat_output import BaseChatOutput
        from ..models.competion_usage import CompetionUsage
        from ..models.doc_reference import DocReference

        choices = []
        for choices_item_data in self.choices:
            choices_item: Dict[str, Any]
            if isinstance(choices_item_data, BaseChatOutput):
                choices_item = choices_item_data.to_dict()
            else:
                choices_item = choices_item_data.to_dict()

            choices.append(choices_item)

        jobs = self.jobs

        docs = []
        for docs_item_data in self.docs:
            docs_item = docs_item_data.to_dict()
            docs.append(docs_item)

        created_at = self.created_at.isoformat()

        id = self.id

        usage: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.usage, Unset):
            usage = self.usage.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "choices": choices,
                "jobs": jobs,
                "docs": docs,
                "created_at": created_at,
                "id": id,
            }
        )
        if usage is not UNSET:
            field_dict["usage"] = usage

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.base_chat_output import BaseChatOutput
        from ..models.competion_usage import CompetionUsage
        from ..models.doc_reference import DocReference

        d = src_dict.copy()
        choices = []
        _choices = d.pop("choices")
        for choices_item_data in _choices:

            def _parse_choices_item(data: object) -> "BaseChatOutput":
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    choices_item_type_0 = BaseChatOutput.from_dict(data)

                    return choices_item_type_0
                except:  # noqa: E722
                    pass
                if not isinstance(data, dict):
                    raise TypeError()
                choices_item_type_1 = BaseChatOutput.from_dict(data)

                return choices_item_type_1

            choices_item = _parse_choices_item(choices_item_data)

            choices.append(choices_item)

        jobs = cast(List[str], d.pop("jobs"))

        docs = []
        _docs = d.pop("docs")
        for docs_item_data in _docs:
            docs_item = DocReference.from_dict(docs_item_data)

            docs.append(docs_item)

        created_at = isoparse(d.pop("created_at"))

        id = d.pop("id")

        _usage = d.pop("usage", UNSET)
        usage: Union[Unset, CompetionUsage]
        if isinstance(_usage, Unset):
            usage = UNSET
        else:
            usage = CompetionUsage.from_dict(_usage)

        message_chat_response = cls(
            choices=choices,
            jobs=jobs,
            docs=docs,
            created_at=created_at,
            id=id,
            usage=usage,
        )

        message_chat_response.additional_properties = d
        return message_chat_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.transition_event_type import TransitionEventType
from ..types import UNSET, Unset

T = TypeVar("T", bound="TransitionEvent")


@_attrs_define
class TransitionEvent:
    """
    Attributes:
        type (TransitionEventType):
        output (Any):
        created_at (datetime.datetime): When this resource was created as UTC date-time
        updated_at (datetime.datetime): When this resource was updated as UTC date-time
    """

    type: TransitionEventType
    output: Any
    created_at: datetime.datetime
    updated_at: datetime.datetime
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        output = self.output

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "output": output,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = TransitionEventType(d.pop("type"))

        output = d.pop("output")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        transition_event = cls(
            type=type,
            output=output,
            created_at=created_at,
            updated_at=updated_at,
        )

        transition_event.additional_properties = d
        return transition_event

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ExecutionInput")


@_attrs_define
class ExecutionInput:
    """The input to the execution"""

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        execution_input = cls()

        execution_input.additional_properties = d
        return execution_input

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.set_step_kind import SetStepKind
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.set_step_set import SetStepSet


T = TypeVar("T", bound="SetStep")


@_attrs_define
class SetStep:
    """
    Attributes:
        kind (SetStepKind): The kind of step Default: SetStepKind.SET.
        set_ (SetStepSet): The value to set
    """

    set_: "SetStepSet"
    kind: SetStepKind = SetStepKind.SET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.set_step_set import SetStepSet

        kind = self.kind.value

        set_ = self.set_.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "kind_": kind,
                "set": set_,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.set_step_set import SetStepSet

        d = src_dict.copy()
        kind = SetStepKind(d.pop("kind_"))

        set_ = SetStepSet.from_dict(d.pop("set"))

        set_step = cls(
            kind=kind,
            set_=set_,
        )

        set_step.additional_properties = d
        return set_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.entries_image_detail import EntriesImageDetail
from ..types import UNSET, Unset

T = TypeVar("T", bound="ChatInputMessagesItemContentType2ItemType1ImageUrl")


@_attrs_define
class ChatInputMessagesItemContentType2ItemType1ImageUrl:
    """The image URL

    Attributes:
        url (str): Image URL or base64 data url (e.g. `data:image/jpeg;base64,<the base64 encoded image>`)
        detail (EntriesImageDetail): Image detail level Default: EntriesImageDetail.AUTO.
    """

    url: str
    detail: EntriesImageDetail = EntriesImageDetail.AUTO
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        url = self.url

        detail = self.detail.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "url": url,
                "detail": detail,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        url = d.pop("url")

        detail = EntriesImageDetail(d.pop("detail"))

        chat_input_messages_item_content_type_2_item_type_1_image_url = cls(
            url=url,
            detail=detail,
        )

        chat_input_messages_item_content_type_2_item_type_1_image_url.additional_properties = d
        return chat_input_messages_item_content_type_2_item_type_1_image_url

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.default_chat_settings import DefaultChatSettings
    from ..models.patch_agent_request_metadata import PatchAgentRequestMetadata


T = TypeVar("T", bound="PatchAgentRequest")


@_attrs_define
class PatchAgentRequest:
    """Payload for patching a agent

    Attributes:
        metadata (Union[Unset, PatchAgentRequestMetadata]):
        name (Union[Unset, str]): For Unicode character safety
            See: https://unicode.org/reports/tr31/
            See: https://www.unicode.org/reports/tr39/#Identifier_Characters Default: ''.
        about (Union[Unset, str]): About the agent Default: ''.
        model (Union[Unset, str]): Model name to use (gpt-4-turbo, gemini-nano etc) Default: ''.
        instructions (Union[List[str], Unset, str]): Instructions for the agent Default: '[]'.
        default_settings (Union[Unset, DefaultChatSettings]): Default settings for the chat session (also used by the
            agent)
    """

    metadata: Union[Unset, "PatchAgentRequestMetadata"] = UNSET
    name: Union[Unset, str] = ""
    about: Union[Unset, str] = ""
    model: Union[Unset, str] = ""
    instructions: Union[List[str], Unset, str] = "[]"
    default_settings: Union[Unset, "DefaultChatSettings"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.default_chat_settings import DefaultChatSettings
        from ..models.patch_agent_request_metadata import PatchAgentRequestMetadata

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        name = self.name

        about = self.about

        model = self.model

        instructions: Union[List[str], Unset, str]
        if isinstance(self.instructions, Unset):
            instructions = UNSET
        elif isinstance(self.instructions, list):
            instructions = self.instructions

        else:
            instructions = self.instructions

        default_settings: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.default_settings, Unset):
            default_settings = self.default_settings.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if name is not UNSET:
            field_dict["name"] = name
        if about is not UNSET:
            field_dict["about"] = about
        if model is not UNSET:
            field_dict["model"] = model
        if instructions is not UNSET:
            field_dict["instructions"] = instructions
        if default_settings is not UNSET:
            field_dict["default_settings"] = default_settings

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.default_chat_settings import DefaultChatSettings
        from ..models.patch_agent_request_metadata import PatchAgentRequestMetadata

        d = src_dict.copy()
        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, PatchAgentRequestMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = PatchAgentRequestMetadata.from_dict(_metadata)

        name = d.pop("name", UNSET)

        about = d.pop("about", UNSET)

        model = d.pop("model", UNSET)

        def _parse_instructions(data: object) -> Union[List[str], Unset, str]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                instructions_type_1 = cast(List[str], data)

                return instructions_type_1
            except:  # noqa: E722
                pass
            return cast(Union[List[str], Unset, str], data)

        instructions = _parse_instructions(d.pop("instructions", UNSET))

        _default_settings = d.pop("default_settings", UNSET)
        default_settings: Union[Unset, DefaultChatSettings]
        if isinstance(_default_settings, Unset):
            default_settings = UNSET
        else:
            default_settings = DefaultChatSettings.from_dict(_default_settings)

        patch_agent_request = cls(
            metadata=metadata,
            name=name,
            about=about,
            model=model,
            instructions=instructions,
            default_settings=default_settings,
        )

        patch_agent_request.additional_properties = d
        return patch_agent_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.execution_status import ExecutionStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.execution_input import ExecutionInput
    from ..models.execution_metadata import ExecutionMetadata


T = TypeVar("T", bound="Execution")


@_attrs_define
class Execution:
    """
    Attributes:
        task_id (str):
        status (ExecutionStatus): The status of the execution
        input_ (ExecutionInput): The input to the execution
        created_at (datetime.datetime): When this resource was created as UTC date-time
        updated_at (datetime.datetime): When this resource was updated as UTC date-time
        id (str):
        output (Union[Unset, Any]): The output of the execution if it succeeded
        error (Union[Unset, str]): The error of the execution if it failed
        metadata (Union[Unset, ExecutionMetadata]):
    """

    task_id: str
    status: ExecutionStatus
    input_: "ExecutionInput"
    created_at: datetime.datetime
    updated_at: datetime.datetime
    id: str
    output: Union[Unset, Any] = UNSET
    error: Union[Unset, str] = UNSET
    metadata: Union[Unset, "ExecutionMetadata"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.execution_input import ExecutionInput
        from ..models.execution_metadata import ExecutionMetadata

        task_id = self.task_id

        status = self.status.value

        input_ = self.input_.to_dict()

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        id = self.id

        output = self.output

        error = self.error

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "task_id": task_id,
                "status": status,
                "input": input_,
                "created_at": created_at,
                "updated_at": updated_at,
                "id": id,
            }
        )
        if output is not UNSET:
            field_dict["output"] = output
        if error is not UNSET:
            field_dict["error"] = error
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.execution_input import ExecutionInput
        from ..models.execution_metadata import ExecutionMetadata

        d = src_dict.copy()
        task_id = d.pop("task_id")

        status = ExecutionStatus(d.pop("status"))

        input_ = ExecutionInput.from_dict(d.pop("input"))

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        id = d.pop("id")

        output = d.pop("output", UNSET)

        error = d.pop("error", UNSET)

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, ExecutionMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = ExecutionMetadata.from_dict(_metadata)

        execution = cls(
            task_id=task_id,
            status=status,
            input_=input_,
            created_at=created_at,
            updated_at=updated_at,
            id=id,
            output=output,
            error=error,
            metadata=metadata,
        )

        execution.additional_properties = d
        return execution

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class UpdateExecutionRequestStatus(str, Enum):
    AWAITING_INPUT = "awaiting_input"
    CANCELLED = "cancelled"
    FAILED = "failed"
    QUEUED = "queued"
    RUNNING = "running"
    STARTING = "starting"
    SUCCEEDED = "succeeded"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class TransitionType(str, Enum):
    CANCELLED = "cancelled"
    ERROR = "error"
    FINISH = "finish"
    FINISH_BRANCH = "finish_branch"
    INIT = "init"
    INIT_BRANCH = "init_branch"
    RESUME = "resume"
    STEP = "step"
    WAIT = "wait"

    def __str__(self) -> str:
        return str(self.value)



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.switch_step_kind import SwitchStepKind
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.case_then import CaseThen


T = TypeVar("T", bound="SwitchStep")


@_attrs_define
class SwitchStep:
    """
    Attributes:
        kind (SwitchStepKind): The kind of step Default: SwitchStepKind.SWITCH.
        switch (List['CaseThen']): The cond tree
    """

    switch: List["CaseThen"]
    kind: SwitchStepKind = SwitchStepKind.SWITCH
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.case_then import CaseThen

        kind = self.kind.value

        switch = []
        for switch_item_data in self.switch:
            switch_item = switch_item_data.to_dict()
            switch.append(switch_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "kind_": kind,
                "switch": switch,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.case_then import CaseThen

        d = src_dict.copy()
        kind = SwitchStepKind(d.pop("kind_"))

        switch = []
        _switch = d.pop("switch")
        for switch_item_data in _switch:
            switch_item = CaseThen.from_dict(switch_item_data)

            switch.append(switch_item)

        switch_step = cls(
            kind=kind,
            switch=switch,
        )

        switch_step.additional_properties = d
        return switch_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import json
import os
from functools import partial, wraps
from importlib import import_module
from typing import Coroutine, TypeVar, Callable, Any, ParamSpec

from .sdk.client import AuthenticatedClient
import julep.sdk.api.default as ops  # This is needed because of the way relative imports work
from .sdk import errors


### GOAL: Convert all autogenerated code into a slightly better interface
# - The generated code is in sdk/api/default and sdk/models
# - each file in sdk/api/default is an operation that contains
#   + a function called `sync_detailed`
#   + a function called `asyncio_detailed`

# - for example, `agents_route_create.py` contains:
#   + def sync_detailed(self, *, client: AuthenticatedClient, body: CreateAgentRequest) -> Response[Agent]:
#   + def asyncio_detailed(self, *, client: AuthenticatedClient, body: CreateAgentRequest) -> Response[Agent]:

# - each file in sdk/models is an `attrs` class that is a model of the API response

# What I want is:
# - a single client that can be used to call all the methods
# - a neat namespaced interface to call the methods

# so the above methods would be called like:
# - julep.agents.create(body=CreateAgentRequest(...))


# Need to get all files from ops module and import them

# Get all the files from the ops module
ops_files = [
    file
    for file in os.listdir(ops.__file__.rstrip("__init__.py"))
    if file.endswith(".py") and file != "__init__.py"
]

# Import all the files
for file in ops_files:
    file_name = file[:-3]
    try:
        import_module(f"julep.sdk.api.default.{file_name}")
    except ImportError:
        breakpoint()
        raise

    setattr(ops, file_name, import_module(f"julep.sdk.api.default.{file_name}"))

# Now we have `ops.agents_route_create` etc.

# These are used to forward the types from the generated code to the decorated functions
T = TypeVar("T", bound=Callable[..., Any])

P = ParamSpec("P")
R = TypeVar("R")


# Decorator to parse the response from the generated code
def parse_response(fn: Callable[P, R]) -> Callable[P, R | dict[str, Any]]:
    @wraps(fn)
    def wrapper(*args: P.args, **kwargs: P.kwargs) -> R | dict[str, Any]:
        #
        # Call the original function
        response = fn(*args, **kwargs)

        # If the response is an error, raise an exception
        if response.status_code.is_client_error or response.status_code.is_server_error:
            raise errors.UnexpectedStatus(response.status_code, response.content)

        else:
            parsed = response.parsed

            # If the response is a model, return it
            if parsed:
                return parsed

            else:
                # Unfortuantely, sometimes the generated code doesn't return a model,
                #   so we need to parse the JSON manually
                return json.loads(response.content)

    return wrapper


# Same decorator but for async functions
def parse_response_async(
    fn: Callable[P, R],
) -> Callable[P, Coroutine[Any, Any, R | dict[str, Any]]]:
    @wraps(fn)
    async def wrapper(*args: P.args, **kwargs: P.kwargs) -> R | dict[str, Any]:
        response = await fn(*args, **kwargs)

        if response.status_code.is_client_error or response.status_code.is_server_error:
            raise errors.UnexpectedStatus(response.status_code, response.content)
        else:
            parsed = response.parsed

            if parsed:
                return parsed
            else:
                return json.loads(response.content)

    return wrapper


# This is the namespace class that will be used to attach the methods to
# so `julep.agents = JulepNamespace("agents", client)`
class JulepNamespace:
    def __init__(self, name: str, client: AuthenticatedClient):
        self.name = name
        self.client = client


class Julep:
    def __init__(
        self,
        *,
        api_key: str,
        base_url: str = "https://api.julep.ai/api",
        **client_kwargs,
    ):
        self.client = AuthenticatedClient(
            token=api_key, base_url=base_url, **client_kwargs
        )

        # Get a list of all the available operations (attributes of the ops object)
        op_names: list[str] = [
            attr for attr in dir(ops)
            # We only want the operations, not the helpers etc.
            if not attr.startswith("_") and "route" in attr
        ]

        # These look like: agents_route_create / agents_route_list etc.
        # The conventions are:
        # - The first part of the name is the resource name (agents, chat, etc.)
        # - The second part of the name is the method (route_create, route_list, etc.)
        # - The `_route_` prefix can be omitted.

        # We want to create a method on the Julep class for each of these that proxies to the ops object
        # But also ensures that the first argument (self.client) is passed through

        namespaces, operations = list(
            zip(*[name.split("_route_") for name in op_names])
        )

        # So now we have: [("agents", "create"), ("agents", "list"), ...]

        # Some namespaces have aliases
        # This is because the API is organized by resource, but the SDK is organized by namespace
        # And we want to allow the user to use the resource name as the namespace name
        namespace_aliases = {
            "agents_docs_search": "agent_docs",
            "user_docs_search": "user_docs",
            "execution_transitions": "transitions",
            "execution_transitions_stream": "transitions",
            "individual_docs": "docs",
            "job": "jobs",
            "task_executions": "executions",
            "tasks_create_or_update": "tasks",
        }

        # Now let's add the namespaces to the Julep class as attributes
        for namespace in namespaces:
            namespace = namespace_aliases.get(namespace, namespace)
            if not hasattr(self, namespace):
                setattr(self, namespace, JulepNamespace(namespace, self.client))

        # so now we have:
        # - `julep.agents`
        # - `julep.chat`
        # - `julep.agent_docs`
        # - `julep.user_docs`
        # - `julep.transitions`
        # - `julep.jobs`
        # - `julep.executions`
        # - `julep.tasks`

        # Next let's add the operations to the Julep class as attributes
        # We need to add both sync and async versions
        # The sync version is called `sync_detailed` and the async version is called `asyncio_detailed`
        # The `_route_` prefix is omitted
        async_prefix = {"a": "asyncio_detailed", "": "sync_detailed"}

        # However in the final client namespace we will add `a..` as prefix to denote the async version
        # for example: `julep.agents.create` and `julep.agents.acreate`
        for prefix, op_name in async_prefix.items():
            for namespace, operation in zip(namespaces, operations):
                # Get ops.agents_route_create
                op = getattr(ops, f"{namespace}_route_{operation}")
                # Get ops.agents_route_create.sync_detailed
                op = getattr(op, op_name)
                # Add the client argument as a partial
                op = partial(op, client=self.client)
                # Decorate with the parse_response decorator
                op = (parse_response_async if (prefix == "a") else parse_response)(op)
                # Prepare the name for the final client namespace
                final_op_name = prefix + operation

                # Get the namespace object
                namespace = namespace_aliases.get(namespace, namespace)
                ns = getattr(self, namespace)

                # Add the operation to the namespace
                if not hasattr(ns, final_op_name):
                    setattr(ns, final_op_name, op)



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.context_overflow_type import ContextOverflowType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.create_session_request_metadata import CreateSessionRequestMetadata


T = TypeVar("T", bound="CreateSessionRequest")


@_attrs_define
class CreateSessionRequest:
    r"""Payload for creating a session

    Attributes:
        situation (str): A specific situation that sets the background for this session Default: '{%- if agent.name
            -%}\nYou are {{agent.name}}.{{\\" \\"}}\n{%- endif -%}\n\n{%- if agent.about -%}\nAbout you:
            {{agent.name}}.{{\\" \\"}}\n{%- endif -%}\n\n{%- if user -%}\nYou are talking to a user\n  {%- if user.name
            -%}{{\\" \\"}} and their name is {{user.name}}\n    {%- if user.about -%}. About the user: {{user.about}}.{%-
            else -%}.{%- endif -%}\n  {%- endif -%}\n{%- endif -%}\n\n{{\\"\n\n\\"}}\n\n{%- if agent.instructions
            -%}\nInstructions:{{\\"\n\\"}}\n  {%- if agent.instructions is string -%}\n
            {{agent.instructions}}{{\\"\n\\"}}\n  {%- else -%}\n    {%- for instruction in agent.instructions -%}\n      -
            {{instruction}}{{\\"\n\\"}}\n    {%- endfor -%}\n  {%- endif -%}\n  {{\\"\n\\"}}\n{%- endif -%}\n\n{%- if tools
            -%}\nTools:{{\\"\n\\"}}\n  {%- for tool in tools -%}\n    {%- if tool.type == \\"function\\" -%}\n      -
            {{tool.function.name}}\n      {%- if tool.function.description -%}: {{tool.function.description}}{%- endif
            -%}{{\\"\n\\"}}\n    {%- else -%}\n      - {{ 0/0 }} {# Error: Other tool types aren\'t supported yet. #}\n
            {%- endif -%}\n  {%- endfor -%}\n{{\\"\n\n\\"}}\n{%- endif -%}\n\n{%- if docs -%}\nRelevant
            documents:{{\\"\n\\"}}\n  {%- for doc in docs -%}\n    {{doc.title}}{{\\"\n\\"}}\n    {%- if doc.content is
            string -%}\n      {{doc.content}}{{\\"\n\\"}}\n    {%- else -%}\n      {%- for snippet in doc.content -%}\n
            {{snippet}}{{\\"\n\\"}}\n      {%- endfor -%}\n    {%- endif -%}\n    {{\\"---\\"}}\n  {%- endfor -%}\n{%- endif
            -%}'.
        render_templates (bool): Render system and assistant message content as jinja templates Default: True.
        token_budget (Union[None, int]): Threshold value for the adaptive context functionality
        context_overflow (Union[ContextOverflowType, None]): Action to start on context window overflow
        user (Union[Unset, str]):
        users (Union[Unset, List[str]]):
        agent (Union[Unset, str]):
        agents (Union[Unset, List[str]]):
        metadata (Union[Unset, CreateSessionRequestMetadata]):
    """

    token_budget: Union[None, int]
    context_overflow: Union[ContextOverflowType, None]
    situation: str = '{%- if agent.name -%}\nYou are {{agent.name}}.{{\\" \\"}}\n{%- endif -%}\n\n{%- if agent.about -%}\nAbout you: {{agent.name}}.{{\\" \\"}}\n{%- endif -%}\n\n{%- if user -%}\nYou are talking to a user\n  {%- if user.name -%}{{\\" \\"}} and their name is {{user.name}}\n    {%- if user.about -%}. About the user: {{user.about}}.{%- else -%}.{%- endif -%}\n  {%- endif -%}\n{%- endif -%}\n\n{{\\"\n\n\\"}}\n\n{%- if agent.instructions -%}\nInstructions:{{\\"\n\\"}}\n  {%- if agent.instructions is string -%}\n    {{agent.instructions}}{{\\"\n\\"}}\n  {%- else -%}\n    {%- for instruction in agent.instructions -%}\n      - {{instruction}}{{\\"\n\\"}}\n    {%- endfor -%}\n  {%- endif -%}\n  {{\\"\n\\"}}\n{%- endif -%}\n\n{%- if tools -%}\nTools:{{\\"\n\\"}}\n  {%- for tool in tools -%}\n    {%- if tool.type == \\"function\\" -%}\n      - {{tool.function.name}}\n      {%- if tool.function.description -%}: {{tool.function.description}}{%- endif -%}{{\\"\n\\"}}\n    {%- else -%}\n      - {{ 0/0 }} {# Error: Other tool types aren\'t supported yet. #}\n    {%- endif -%}\n  {%- endfor -%}\n{{\\"\n\n\\"}}\n{%- endif -%}\n\n{%- if docs -%}\nRelevant documents:{{\\"\n\\"}}\n  {%- for doc in docs -%}\n    {{doc.title}}{{\\"\n\\"}}\n    {%- if doc.content is string -%}\n      {{doc.content}}{{\\"\n\\"}}\n    {%- else -%}\n      {%- for snippet in doc.content -%}\n        {{snippet}}{{\\"\n\\"}}\n      {%- endfor -%}\n    {%- endif -%}\n    {{\\"---\\"}}\n  {%- endfor -%}\n{%- endif -%}'
    render_templates: bool = True
    user: Union[Unset, str] = UNSET
    users: Union[Unset, List[str]] = UNSET
    agent: Union[Unset, str] = UNSET
    agents: Union[Unset, List[str]] = UNSET
    metadata: Union[Unset, "CreateSessionRequestMetadata"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.create_session_request_metadata import (
            CreateSessionRequestMetadata,
        )

        situation = self.situation

        render_templates = self.render_templates

        token_budget: Union[None, int]
        token_budget = self.token_budget

        context_overflow: Union[None, str]
        if isinstance(self.context_overflow, ContextOverflowType):
            context_overflow = self.context_overflow.value
        else:
            context_overflow = self.context_overflow

        user = self.user

        users: Union[Unset, List[str]] = UNSET
        if not isinstance(self.users, Unset):
            users = self.users

        agent = self.agent

        agents: Union[Unset, List[str]] = UNSET
        if not isinstance(self.agents, Unset):
            agents = self.agents

        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "situation": situation,
                "render_templates": render_templates,
                "token_budget": token_budget,
                "context_overflow": context_overflow,
            }
        )
        if user is not UNSET:
            field_dict["user"] = user
        if users is not UNSET:
            field_dict["users"] = users
        if agent is not UNSET:
            field_dict["agent"] = agent
        if agents is not UNSET:
            field_dict["agents"] = agents
        if metadata is not UNSET:
            field_dict["metadata"] = metadata

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.create_session_request_metadata import (
            CreateSessionRequestMetadata,
        )

        d = src_dict.copy()
        situation = d.pop("situation")

        render_templates = d.pop("render_templates")

        def _parse_token_budget(data: object) -> Union[None, int]:
            if data is None:
                return data
            return cast(Union[None, int], data)

        token_budget = _parse_token_budget(d.pop("token_budget"))

        def _parse_context_overflow(data: object) -> Union[ContextOverflowType, None]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                context_overflow_type_0 = ContextOverflowType(data)

                return context_overflow_type_0
            except:  # noqa: E722
                pass
            return cast(Union[ContextOverflowType, None], data)

        context_overflow = _parse_context_overflow(d.pop("context_overflow"))

        user = d.pop("user", UNSET)

        users = cast(List[str], d.pop("users", UNSET))

        agent = d.pop("agent", UNSET)

        agents = cast(List[str], d.pop("agents", UNSET))

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, CreateSessionRequestMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = CreateSessionRequestMetadata.from_dict(_metadata)

        create_session_request = cls(
            situation=situation,
            render_templates=render_templates,
            token_budget=token_budget,
            context_overflow=context_overflow,
            user=user,
            users=users,
            agent=agent,
            agents=agents,
            metadata=metadata,
        )

        create_session_request.additional_properties = d
        return create_session_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.chat_token_log_prob import ChatTokenLogProb


T = TypeVar("T", bound="LogProbResponse")


@_attrs_define
class LogProbResponse:
    """
    Attributes:
        content (Union[List['ChatTokenLogProb'], None]): The log probabilities of the tokens
    """

    content: Union[List["ChatTokenLogProb"], None]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.chat_token_log_prob import ChatTokenLogProb

        content: Union[List[Dict[str, Any]], None]
        if isinstance(self.content, list):
            content = []
            for content_type_0_item_data in self.content:
                content_type_0_item = content_type_0_item_data.to_dict()
                content.append(content_type_0_item)

        else:
            content = self.content

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "content": content,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.chat_token_log_prob import ChatTokenLogProb

        d = src_dict.copy()

        def _parse_content(data: object) -> Union[List["ChatTokenLogProb"], None]:
            if data is None:
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                content_type_0 = []
                _content_type_0 = data
                for content_type_0_item_data in _content_type_0:
                    content_type_0_item = ChatTokenLogProb.from_dict(
                        content_type_0_item_data
                    )

                    content_type_0.append(content_type_0_item)

                return content_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List["ChatTokenLogProb"], None], data)

        content = _parse_content(d.pop("content"))

        log_prob_response = cls(
            content=content,
        )

        log_prob_response.additional_properties = d
        return log_prob_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CompetionUsage")


@_attrs_define
class CompetionUsage:
    """Usage statistics for the completion request

    Attributes:
        completion_tokens (Union[Unset, int]): Number of tokens in the generated completion
        prompt_tokens (Union[Unset, int]): Number of tokens in the prompt
        total_tokens (Union[Unset, int]): Total number of tokens used in the request (prompt + completion)
    """

    completion_tokens: Union[Unset, int] = UNSET
    prompt_tokens: Union[Unset, int] = UNSET
    total_tokens: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        completion_tokens = self.completion_tokens

        prompt_tokens = self.prompt_tokens

        total_tokens = self.total_tokens

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if completion_tokens is not UNSET:
            field_dict["completion_tokens"] = completion_tokens
        if prompt_tokens is not UNSET:
            field_dict["prompt_tokens"] = prompt_tokens
        if total_tokens is not UNSET:
            field_dict["total_tokens"] = total_tokens

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        completion_tokens = d.pop("completion_tokens", UNSET)

        prompt_tokens = d.pop("prompt_tokens", UNSET)

        total_tokens = d.pop("total_tokens", UNSET)

        competion_usage = cls(
            completion_tokens=completion_tokens,
            prompt_tokens=prompt_tokens,
            total_tokens=total_tokens,
        )

        competion_usage.additional_properties = d
        return competion_usage

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.hybrid_doc_search_request import HybridDocSearchRequest
    from ..models.text_only_doc_search_request import TextOnlyDocSearchRequest
    from ..models.vector_doc_search_request import VectorDocSearchRequest


T = TypeVar("T", bound="DocsSearchRouteSearchBody")


@_attrs_define
class DocsSearchRouteSearchBody:
    """
    Attributes:
        body (Union['HybridDocSearchRequest', 'TextOnlyDocSearchRequest', 'VectorDocSearchRequest']):
    """

    body: Union[
        "HybridDocSearchRequest", "TextOnlyDocSearchRequest", "VectorDocSearchRequest"
    ]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.hybrid_doc_search_request import HybridDocSearchRequest
        from ..models.text_only_doc_search_request import TextOnlyDocSearchRequest
        from ..models.vector_doc_search_request import VectorDocSearchRequest

        body: Dict[str, Any]
        if isinstance(self.body, VectorDocSearchRequest):
            body = self.body.to_dict()
        elif isinstance(self.body, TextOnlyDocSearchRequest):
            body = self.body.to_dict()
        else:
            body = self.body.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "body": body,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.hybrid_doc_search_request import HybridDocSearchRequest
        from ..models.text_only_doc_search_request import TextOnlyDocSearchRequest
        from ..models.vector_doc_search_request import VectorDocSearchRequest

        d = src_dict.copy()

        def _parse_body(
            data: object,
        ) -> Union[
            "HybridDocSearchRequest",
            "TextOnlyDocSearchRequest",
            "VectorDocSearchRequest",
        ]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                body_type_0 = VectorDocSearchRequest.from_dict(data)

                return body_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                body_type_1 = TextOnlyDocSearchRequest.from_dict(data)

                return body_type_1
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            body_type_2 = HybridDocSearchRequest.from_dict(data)

            return body_type_2

        body = _parse_body(d.pop("body"))

        docs_search_route_search_body = cls(
            body=body,
        )

        docs_search_route_search_body.additional_properties = d
        return docs_search_route_search_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.chat_settings_logit_bias import ChatSettingsLogitBias
    from ..models.schema_completion_response_format import (
        SchemaCompletionResponseFormat,
    )
    from ..models.simple_completion_response_format import (
        SimpleCompletionResponseFormat,
    )


T = TypeVar("T", bound="ChatSettings")


@_attrs_define
class ChatSettings:
    """
    Attributes:
        stream (bool): Indicates if the server should stream the response as it's generated Default: False.
        stop (List[str]): Up to 4 sequences where the API will stop generating further tokens.
        frequency_penalty (Union[Unset, float]): Number between -2.0 and 2.0. Positive values penalize new tokens based
            on their existing frequency in the text so far, decreasing the model's likelihood to repeat the same line
            verbatim.
        presence_penalty (Union[Unset, float]): Number between -2.0 and 2.0. Positive values penalize new tokens based
            on their existing frequency in the text so far, decreasing the model's likelihood to repeat the same line
            verbatim.
        temperature (Union[Unset, float]): What sampling temperature to use, between 0 and 2. Higher values like 0.8
            will make the output more random, while lower values like 0.2 will make it more focused and deterministic.
        top_p (Union[Unset, float]): Defaults to 1 An alternative to sampling with temperature, called nucleus sampling,
            where the model considers the results of the tokens with top_p probability mass. So 0.1 means only the tokens
            comprising the top 10% probability mass are considered.  We generally recommend altering this or temperature but
            not both.
        repetition_penalty (Union[Unset, float]): Number between 0 and 2.0. 1.0 is neutral and values larger than that
            penalize new tokens based on their existing frequency in the text so far, decreasing the model's likelihood to
            repeat the same line verbatim.
        length_penalty (Union[Unset, float]): Number between 0 and 2.0. 1.0 is neutral and values larger than that
            penalize number of tokens generated.
        min_p (Union[Unset, float]): Minimum probability compared to leading token to be considered
        model (Union[Unset, str]): For Unicode character safety
            See: https://unicode.org/reports/tr31/
            See: https://www.unicode.org/reports/tr39/#Identifier_Characters
        seed (Union[Unset, int]): If specified, the system will make a best effort to sample deterministically for that
            particular seed value
        max_tokens (Union[Unset, int]): The maximum number of tokens to generate in the chat completion
        logit_bias (Union[Unset, ChatSettingsLogitBias]): Modify the likelihood of specified tokens appearing in the
            completion
        response_format (Union['SchemaCompletionResponseFormat', 'SimpleCompletionResponseFormat', Unset]): Response
            format (set to `json_object` to restrict output to JSON)
        agent (Union[Unset, str]):
    """

    stop: List[str]
    stream: bool = False
    frequency_penalty: Union[Unset, float] = UNSET
    presence_penalty: Union[Unset, float] = UNSET
    temperature: Union[Unset, float] = UNSET
    top_p: Union[Unset, float] = UNSET
    repetition_penalty: Union[Unset, float] = UNSET
    length_penalty: Union[Unset, float] = UNSET
    min_p: Union[Unset, float] = UNSET
    model: Union[Unset, str] = UNSET
    seed: Union[Unset, int] = UNSET
    max_tokens: Union[Unset, int] = UNSET
    logit_bias: Union[Unset, "ChatSettingsLogitBias"] = UNSET
    response_format: Union[
        "SchemaCompletionResponseFormat", "SimpleCompletionResponseFormat", Unset
    ] = UNSET
    agent: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.chat_settings_logit_bias import ChatSettingsLogitBias
        from ..models.schema_completion_response_format import (
            SchemaCompletionResponseFormat,
        )
        from ..models.simple_completion_response_format import (
            SimpleCompletionResponseFormat,
        )

        stream = self.stream

        stop = self.stop

        frequency_penalty = self.frequency_penalty

        presence_penalty = self.presence_penalty

        temperature = self.temperature

        top_p = self.top_p

        repetition_penalty = self.repetition_penalty

        length_penalty = self.length_penalty

        min_p = self.min_p

        model = self.model

        seed = self.seed

        max_tokens = self.max_tokens

        logit_bias: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.logit_bias, Unset):
            logit_bias = self.logit_bias.to_dict()

        response_format: Union[Dict[str, Any], Unset]
        if isinstance(self.response_format, Unset):
            response_format = UNSET
        elif isinstance(self.response_format, SimpleCompletionResponseFormat):
            response_format = self.response_format.to_dict()
        else:
            response_format = self.response_format.to_dict()

        agent = self.agent

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "stream": stream,
                "stop": stop,
            }
        )
        if frequency_penalty is not UNSET:
            field_dict["frequency_penalty"] = frequency_penalty
        if presence_penalty is not UNSET:
            field_dict["presence_penalty"] = presence_penalty
        if temperature is not UNSET:
            field_dict["temperature"] = temperature
        if top_p is not UNSET:
            field_dict["top_p"] = top_p
        if repetition_penalty is not UNSET:
            field_dict["repetition_penalty"] = repetition_penalty
        if length_penalty is not UNSET:
            field_dict["length_penalty"] = length_penalty
        if min_p is not UNSET:
            field_dict["min_p"] = min_p
        if model is not UNSET:
            field_dict["model"] = model
        if seed is not UNSET:
            field_dict["seed"] = seed
        if max_tokens is not UNSET:
            field_dict["max_tokens"] = max_tokens
        if logit_bias is not UNSET:
            field_dict["logit_bias"] = logit_bias
        if response_format is not UNSET:
            field_dict["response_format"] = response_format
        if agent is not UNSET:
            field_dict["agent"] = agent

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.chat_settings_logit_bias import ChatSettingsLogitBias
        from ..models.schema_completion_response_format import (
            SchemaCompletionResponseFormat,
        )
        from ..models.simple_completion_response_format import (
            SimpleCompletionResponseFormat,
        )

        d = src_dict.copy()
        stream = d.pop("stream")

        stop = cast(List[str], d.pop("stop"))

        frequency_penalty = d.pop("frequency_penalty", UNSET)

        presence_penalty = d.pop("presence_penalty", UNSET)

        temperature = d.pop("temperature", UNSET)

        top_p = d.pop("top_p", UNSET)

        repetition_penalty = d.pop("repetition_penalty", UNSET)

        length_penalty = d.pop("length_penalty", UNSET)

        min_p = d.pop("min_p", UNSET)

        model = d.pop("model", UNSET)

        seed = d.pop("seed", UNSET)

        max_tokens = d.pop("max_tokens", UNSET)

        _logit_bias = d.pop("logit_bias", UNSET)
        logit_bias: Union[Unset, ChatSettingsLogitBias]
        if isinstance(_logit_bias, Unset):
            logit_bias = UNSET
        else:
            logit_bias = ChatSettingsLogitBias.from_dict(_logit_bias)

        def _parse_response_format(
            data: object,
        ) -> Union[
            "SchemaCompletionResponseFormat", "SimpleCompletionResponseFormat", Unset
        ]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_format_type_0 = SimpleCompletionResponseFormat.from_dict(data)

                return response_format_type_0
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            response_format_type_1 = SchemaCompletionResponseFormat.from_dict(data)

            return response_format_type_1

        response_format = _parse_response_format(d.pop("response_format", UNSET))

        agent = d.pop("agent", UNSET)

        chat_settings = cls(
            stream=stream,
            stop=stop,
            frequency_penalty=frequency_penalty,
            presence_penalty=presence_penalty,
            temperature=temperature,
            top_p=top_p,
            repetition_penalty=repetition_penalty,
            length_penalty=length_penalty,
            min_p=min_p,
            model=model,
            seed=seed,
            max_tokens=max_tokens,
            logit_bias=logit_bias,
            response_format=response_format,
            agent=agent,
        )

        chat_settings.additional_properties = d
        return chat_settings

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Dict,
    List,
    Optional,
    TextIO,
    Tuple,
    Type,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.yield_step_arguments_type_1 import YieldStepArgumentsType1
from ..models.yield_step_kind import YieldStepKind
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.yield_step_arguments_type_0 import YieldStepArgumentsType0


T = TypeVar("T", bound="YieldStep")


@_attrs_define
class YieldStep:
    """
    Attributes:
        kind (YieldStepKind): The kind of step Default: YieldStepKind.YIELD.
        workflow (str): The subworkflow to run.
            VALIDATION: Should resolve to a defined subworkflow.
        arguments (Union['YieldStepArgumentsType0', YieldStepArgumentsType1]): The input parameters for the subworkflow
            (defaults to last step output) Default: YieldStepArgumentsType1.VALUE_0.
    """

    workflow: str
    kind: YieldStepKind = YieldStepKind.YIELD
    arguments: Union["YieldStepArgumentsType0", YieldStepArgumentsType1] = (
        YieldStepArgumentsType1.VALUE_0
    )
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.yield_step_arguments_type_0 import YieldStepArgumentsType0

        kind = self.kind.value

        workflow = self.workflow

        arguments: Union[Dict[str, Any], str]
        if isinstance(self.arguments, YieldStepArgumentsType0):
            arguments = self.arguments.to_dict()
        else:
            arguments = self.arguments.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "kind_": kind,
                "workflow": workflow,
                "arguments": arguments,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.yield_step_arguments_type_0 import YieldStepArgumentsType0

        d = src_dict.copy()
        kind = YieldStepKind(d.pop("kind_"))

        workflow = d.pop("workflow")

        def _parse_arguments(
            data: object,
        ) -> Union["YieldStepArgumentsType0", YieldStepArgumentsType1]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                arguments_type_0 = YieldStepArgumentsType0.from_dict(data)

                return arguments_type_0
            except:  # noqa: E722
                pass
            if not isinstance(data, str):
                raise TypeError()
            arguments_type_1 = YieldStepArgumentsType1(data)

            return arguments_type_1

        arguments = _parse_arguments(d.pop("arguments"))

        yield_step = cls(
            kind=kind,
            workflow=workflow,
            arguments=arguments,
        )

        yield_step.additional_properties = d
        return yield_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
